from __future__ import annotations

from typing import List

from .DirectDampingByFrequencyComponent import DirectDampingByFrequencyComponent

DirectDampingByFrequencyComponentArray = List[DirectDampingByFrequencyComponent]
